#define OLC_PGE_APPLICATION
#include "olcPixelGameEngine.h"
#define OLC_PGEX_POPUPMENU
#include "olcPGEX_PopupMenu.h"
#define OLC_PGEX_SHNFile
#include "SHNFileDecryptor.h"
#define OLC_PGEX_TRANSFORMEDVIEW
#include "olcPGEX_TransformedView.h"
#define OLC_PGEX_QUICKGUI
#include "olcPGEX_QuickGUI.h"
#include "FiestaOnlineEditor.h"
#include "ActionIDs.h"
#include "ItemEditor.h"

FiestaOnlineEditor::FiestaOnlineEditor()
{
	// Name your application
	sAppName = "Fiesta Online Editor";
}

bool FiestaOnlineEditor::OnUserCreate(){
	CreateLayer();
	EnableLayer(1,true);

	menu.SetTable(1,6);
	menu["Save"];
	menu["Items"].SetTable(1,4);
	menu["Items"]["Item Editor"].SetID(LOAD_ITEM_EDITOR);
	menu["Items"]["Drop Groups"];
	menu["Items"]["Icons"];
	menu["Mobs"];
	menu["Abilities"];
	menu["Buffs"];
	menu["Reload"].SetID(RELOAD_FILE_DIALOG);
	menu.Build();
	sprMenu = new Sprite("assets/RetroMenu.png");

	selectedPath.push_back("");

	utils::datafile::Read(config,"assets/program.txt");
	InitializePathSelection();

	return true;
}

bool FiestaOnlineEditor::OnUserUpdate(float fElapsedTime){
	lastPressedTime=std::max(0.f,lastPressedTime-fElapsedTime);
	if (GetKey(UP).bHeld&&lastPressedTime==0){
		if(GetKey(UP).bPressed){
			lastPressedTime=0.2;
		} else {
			lastPressedTime=0.03;
		}
		manager.OnUp();
	}
	if (GetKey(DOWN).bHeld&&lastPressedTime==0){
		if(GetKey(DOWN).bPressed){
			lastPressedTime=0.2;
		} else {
			lastPressedTime=0.03;
		}
		manager.OnDown();
	}
	if (GetKey(LEFT).bHeld&&lastPressedTime==0){
		if(GetKey(LEFT).bPressed){
			lastPressedTime=0.2;
		} else {
			lastPressedTime=0.03;
		}
		manager.OnLeft();
	}
	if (GetKey(RIGHT).bHeld&&lastPressedTime==0){
		if(GetKey(RIGHT).bPressed){
			lastPressedTime=0.2;
		} else {
			lastPressedTime=0.03;
		}
		manager.OnRight();
	}
	if (GetKey(X).bPressed){
		if(appState!=LOADFOLDER&&appState!=LOADFILE){
			manager.OnBack();
		}
	}
	if (GetKey(Z).bPressed){
		popup::Menu*selected;
		if((selected=manager.OnConfirm())!=nullptr){
			std::string sLastAction = 
				"Selected: " + selected->GetName() + 
				" ID: " + std::to_string(selected->GetID());
			std::cout<<sLastAction<<std::endl;
			switch(selected->GetID()){
				case RELOAD_FILE_DIALOG:{
				}break;
				case CHANGE_LOAD_PATH:{
					selectedPath.push_back(selectedPath.back()+"/"+selected->GetName());
					if(appState==LOADFOLDER){
						dialog.ChangePath(selectedPath.back());
						manager.Open(&dialog.GetMenu());
					} else {
						filePickerDialog.ChangePath(selectedPath.back());
						manager.Open(&filePickerDialog.GetMenu());
					}
				}break;
				case GO_BACK:{
					selectedPath.pop_back();
					if(appState==LOADFOLDER){
						dialog.ChangePath(selectedPath.back());
						manager.Open(&dialog.GetMenu());
					} else {
						filePickerDialog.ChangePath(selectedPath.back());
						manager.Open(&filePickerDialog.GetMenu());
					}
				}break;
				case SELECTED_FOLDER:{
					std::cout<<selectedPath.back()<<std::endl;
					if(pickFileState==SHINEDIR){
						config["DefaultPath"].SetString(selectedPath.back());
					} else
					if(pickFileState==CLIENTDIR){
						config["ClientPath"].SetString(selectedPath.back());
					} else
					if(pickFileState==IMAGEMAGICKEXE){
						config["ImageMagickEXE"].SetString(selectedPath.back()+"/"+selected->GetName());
					}
					utils::datafile::Write(config,"assets/program.txt");
					manager.Close();
					InitializePathSelection();
				}break;
				case LOAD_ITEM_EDITOR:{
					bool reloadImages=true;
					if(config["ItemEditorImagesCached"].GetInt()==1){
						reloadImages=false;
					}
					ItemEditor*itemEditor=new ItemEditor(this,"Item Editor: "+config["DefaultPath"].GetString(),{32,32},GetScreenSize()-vi2d{64,64},reloadImages);
					CreateWindow(itemEditor);
					itemEditor->Load(config["DefaultPath"].GetString(),config["ClientPath"].GetString(),config["ImageMagickEXE"].GetString());
					manager.Close();
					menuOpened=false;
				}break;
			}
		}
	}

	if(GetKey(ESCAPE).bPressed&&!menuOpened){
		menuOpened=true;
		manager.Open(&menu);
	}else
	if(GetKey(ESCAPE).bPressed&&menuOpened){
		menuOpened=false;
		manager.Close();
	}


	SetDrawTarget(1);
	Clear(olc::VERY_DARK_CYAN);
	SetDrawTarget(nullptr);
	Clear(olc::BLANK);
	SetDrawTarget(1);
	//BACKGROUND DRAWING

	if(appState==LOADFOLDER||appState==LOADFILE){
		switch(pickFileState){
			case SHINEDIR:{
				DrawString({2,2},"Choose the Shine file directory:");
			}break;
			case CLIENTDIR:{
				DrawString({2,2},"Choose the Client file directory:");
			}break;
			case IMAGEMAGICKEXE:{
				DrawString({2,2},"Choose the Image Magick Executable:");
			}break;
		}
	}

	bool internalMouseFocus=false;
	for(Window*w:windows){
		if(!w->IsFocusedWindow()){
			if(!menuOpened){
				w->InternalUpdate(this,fElapsedTime);
			}
			w->Draw(this);
		}
	}

	for(Window*w:windows){
		if(w->IsFocusedWindow()){
			if(!menuOpened){
				if(w->IsInBounds(GetMousePos())){
					w->InternalMouseFocus(this);
					w->MouseFocus(this);
					internalMouseFocus=true;
				}
			}
			if(!menuOpened){
				w->InternalUpdate(this,fElapsedTime);
			}
			w->Draw(this);
		}
	}

	if(!menuOpened){
		for(auto it=windows.begin();it!=windows.end();++it){ //A second iteration through the windows because windows without focus should only have priority after a window with focus.
			Window*w=*it;
			if(!internalMouseFocus){
				if(!w->IsFocusedWindow()){
					if(w->IsInBounds(GetMousePos())){
						w->InternalMouseFocus(this);
						internalMouseFocus=true;
						if(w->IsFocusedWindow()){//The focused window has changed!
							windows.push_back(w);//HACK ALERT! Whenever we are iterating through the windows list we could potentially add an item to the end of it whenever it becomes focused to keep layering. Because of this we don't want to reallocate during another push_back. This ensures that will never occur.
							it=windows.erase(it);
						}
						w->MouseFocus(this);
					}
				}
			}
		}
	}

	bool focusErased=false;
	std::erase_if(windows,[&](Window*w){
		if(w->IsClosed()){
			if(w->IsFocusedWindow()){
				focusErased=true;
			}
			w->Cleanup();
			delete w;
			return true;
		} else {
			return false;
		}
	});

	if(focusErased&&windows.size()>0){
		Window::SetFocusedWindow(this,windows.back());
	}

	SetDrawTarget(nullptr);
	//FOREGROUND DRAWING
	manager.Draw(sprMenu,{64,64});

	return true;
}

void FiestaOnlineEditor::CreateWindow(Window*window){
	for(Window*w:windows){
		w->InternalRefresh(this);
	}
	window->InternalRefresh(this);
	windows.push_back(window);
	windows.reserve(windows.size()+1);//HACK ALERT! Whenever we are iterating through the windows list we could potentially add an item to the end of it whenever it becomes focused to keep layering. Because of this we don't want to reallocate during another push_back. This ensures that will never occur.
}

void FiestaOnlineEditor::InitializePathSelection(){
	if(!config.HasProperty("DefaultPath")){
		pickFileState=SHINEDIR;
		manager.Open(&dialog.GetMenu());
	} else 
	if(!config.HasProperty("ClientPath")){
		pickFileState=CLIENTDIR;
		manager.Open(&dialog.GetMenu());
	} else 
	if(!config.HasProperty("ImageMagickEXE")){
		pickFileState=IMAGEMAGICKEXE;
		appState=LOADFILE;
		manager.Open(&filePickerDialog.GetMenu());
	} else {
		//We have all properties we need. Start using program.
		appState=MENUSELECT;
		manager.Open(&menu);
	}
}

int main()
{
	FiestaOnlineEditor demo;
	if (demo.Construct(960, 360, 2, 2))
		demo.Start();
	return 0;
}